package com.gic.haoban.manage.service.task;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.manage.api.dto.ClerkMainStoreRelatedDTO;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.ExternalUserDTO;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.ClerkMainStoreRelatedApiService;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.pojo.ExternalUserPojo;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.mq.sdk.GicMQClient;

@Service
public class FriendSyncOperation implements BaseSyncOperation {
	private static final Logger logger = LoggerFactory.getLogger(FriendSyncOperation.class);
	@Autowired
	private PreDealService preDealService;
	@Autowired
	private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
	@Autowired
	private SyncTaskService syncTaskService;
	@Autowired
	private ClerkMainStoreRelatedApiService clerkMainStoreRelatedApiService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private StaffClerkRelationService staffClerkRelationService;
	@Override
	public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
		String reason = "";
		boolean dealFlag = true;
		String wxEnterpriseId = dataPre.getWxEnterpriseId();
		String wxUserId = dataPre.getpDataId();
		String taskId = dealParamMqDTO.getTaskId();
		String externalUserId = dataPre.getDataId();
		StaffDTO staff = staffApiService.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);
		if(staff == null){
			preDealService.updateStatusByDataId(taskId, externalUserId, PreDealStatusEnum.exception.getVal(), "成员不存在");
			checkStaffTask(taskId, wxUserId);
			return;
		}
		ClerkMainStoreRelatedDTO mainStore = clerkMainStoreRelatedApiService.getWxEnterpriseIdAndStaffId(wxEnterpriseId, staff.getStaffId());
		if(mainStore == null){
			preDealService.updateStatusByDataId(taskId, externalUserId, PreDealStatusEnum.exception.getVal(), "主门店不存在");
			checkStaffTask(taskId, wxUserId);
			return;
		}
		StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationService.getOneBindByStoreId(staff.getStaffId(),mainStore.getStoreId());
		if(staffClerkRelationDTO == null){
			preDealService.updateStatusByDataId(taskId, externalUserId, PreDealStatusEnum.exception.getVal(), "未绑定导购");
			checkStaffTask(taskId, wxUserId);
			return;
		}
		ExternalUserDTO pojo = memberUnionidRelatedApiService.getExternalUseridInfo(wxEnterpriseId, wxUserId, externalUserId);
		if(pojo == null){
			preDealService.updateStatusByDataId(taskId, externalUserId, PreDealStatusEnum.exception.getVal(), "好友不存在");
			checkStaffTask(taskId, wxUserId);
			return;
		}
		pojo.setEnterpriseId(staffClerkRelationDTO.getEnterpriseId());
		pojo.setStoreId(mainStore.getStoreId());
		pojo.setClerkId(staffClerkRelationDTO.getClerkId());
		if(pojo != null){
			dealSuccess(taskId, externalUserId,JSON.toJSONString(pojo),wxEnterpriseId);
		}
		try {
			checkStaffTask(taskId, wxUserId);
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("同步失败：{},{}", JSONObject.toJSONString(dataPre),e);
            reason = "第三方好友处理异常";
            dealFlag = false;
		}finally {
			if (!dealFlag) {
               dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
           } 
		}
	}

	@Override
	public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
		preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
	}

	@Override
	public void dealSuccess(String taskId, String dataId, String dataContent, String wxEnterpriseId) {
		preDealService.updateFriendStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal(), "成功",dataContent);

	}

	public void checkStaffTask(String taskId,String wxUserId) {
		boolean flag =  preDealService.checkStaffTaskByTaskIdAndPDataId(taskId,wxUserId);
		logger.info("checkStaffTask={}",flag);
		if(flag){
			List<TabHaobanPreDealLog> list = preDealService.listByPDataIdAndTaskId(taskId, wxUserId);
			if(CollectionUtils.isNotEmpty(list)){
				Map<String,ExternalUserDTO> map = new HashMap<>();
				List<TabHaobanPreDealLog> dealLogList = new ArrayList<>();
				for (TabHaobanPreDealLog tabHaobanPreDealLog : list) {
					String dataContent = tabHaobanPreDealLog.getDataContent();
					if(StringUtils.isNotBlank(dataContent)){
						ExternalUserDTO json = JSON.parseObject(dataContent, ExternalUserDTO.class);
						
						String addCreateTime = json.getAddCreateTime();
						String name = json.getName();
						String key = addCreateTime + name;
						String unionId = json.getUnionId();
						String clerkId = json.getClerkId();
						String enterpriseId = json.getEnterpriseId();
						String memberId = json.getMemberId();
						String storeId = json.getStoreId();
						String wxEnterpriseId = json.getWxEnterpriseId();
						
						ExternalUserDTO dto = map.get(key);
						if(dto != null){
							if(tabHaobanPreDealLog.getDataType() == SyncTaskStatusEnum.self_friend_sync.getVal()){
								dto.setUnionId(unionId);
								dto.setClerkId(clerkId);
								dto.setEnterpriseId(enterpriseId);
								dto.setMemberId(memberId);
								dto.setStoreId(storeId);
							}else if(tabHaobanPreDealLog.getDataType() == SyncTaskStatusEnum.friend_sync.getVal()){
								dto.setExternalUserId(tabHaobanPreDealLog.getDataId());
							}
							TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
				            dealLog.setDataId(key);
				            dealLog.setpDataId(wxUserId);
				            dealLog.setDataType(PreDealTypeEnum.friend_merge.getVal());
				            dealLog.setStatusFlag(0);
				            dealLog.setTaskId(taskId);
				            dealLog.setDataContent(JSONObject.toJSONString(dto));
				            dealLog.setWxEnterpriseId(wxEnterpriseId);
				            dealLogList.add(dealLog);
						}else{
							ExternalUserDTO dto2 = JSON.parseObject(dataContent, ExternalUserDTO.class);
							map.put(key, dto2);
						}
					}
				}
				logger.info("dealLogList={}",JSON.toJSONString(dealLogList));
				if(CollectionUtils.isNotEmpty(dealLogList)){
					preDealService.insert(dealLogList);
					HashSet<String> hashSet = (HashSet<String>) dealLogList.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
			        dealDepartmentToMq(taskId, hashSet, SyncTaskStatusEnum.merge_friend_sync);
				}
			}
		}
	}

	@Override
	public void checkDepartmentTask(String taskId) {
		
	}
	
	/**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        dealList.forEach(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
            try {
                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
            } catch (Exception e) {
                logger.info("发送失败:{},{}", taskId, relationId);
                e.printStackTrace();
            }
        });
    }

}
